const path = require('path');
const MenuHelper = require('./helpers/menu.js');

function Menu(Manager) {
  const self = this;
  self.Manager = Manager;

  self.initialized = false;
  self.instance = null;
  self.generate = null;
  self.menuTemplate = [];
  self.contextMenu = null;

  self.analyticsCategory = 'app-menu';

  MenuHelper.init(self);
  // self.item = MenuHelper.item;
  // self.insert = MenuHelper.insert;
}

Menu.prototype.init = function (options) {
  const self = this;

  return new Promise(function(resolve, reject) {
    const Manager = self.Manager;
    const { app, Menu } = Manager.libraries.electron;
    const appPath = app.getAppPath();

    app.whenReady().then(() => {
      self.instance = null;
      self.menuTemplate = [];
      self.generate = null;
      self.contextMenu = null;

      self.generateDefault();

      try {
        self.generate = require(path.join(appPath, 'electron-manager', 'app-menu.js'));
        self.generate(self);
      } catch (e) {
        self.menuTemplate = [];
        console.error('Failed to build from template', e);
      }

      self.dedupe();

      self.contextMenu = Menu.buildFromTemplate(self.menuTemplate);

      Menu.setApplicationMenu(self.contextMenu);
      self.instance = Menu.getApplicationMenu()

      // Not triggered :(
      // https://stackoverflow.com/questions/55009560/electron-listen-to-menu-will-show-event
      // self.instance.on('menu-will-show', (event) => {
      //   console.log('---instance show');
      //   // Manager.analytics().event({category: self.analyticsCategory, action: 'open'})
      // })
      //
      // self.instance.on('menu-will-close', (event) => {
      //   console.log('---instance close');
      //   // Manager.analytics().event({category: self.analyticsCategory, action: 'close'})
      // })

      self.initialized = true;
      return resolve(self);
    })
  });
};

Menu.prototype.generateDefault = function () {
  const self = this;
  const Manager = self.Manager;
  const { app, Menu, shell } = Manager.libraries.electron;

  const resolvedDeveloper = Manager.resolveDeveloper();
  const isMac = process.platform === 'darwin';
  const isWin = process.platform === 'win32';
  // let resolvedDeveloper = false;

  let updaterStatus = {};
  try {
    updaterStatus = Manager.libraries.updater.status;
  } catch (e) {

  }

  // Set default
  self.menuTemplate = [
    // Main
    {
      id: 'main',
      label: app.name,
      submenu: [
        {
          id: 'main/about',
          label: `&About ${app.name}`,
          role: `about`,
          // selector: 'orderFrontStandardAboutPanel:'
        },
        {
          id: 'main/update-check',
          label: `Check for &Updates`,
          visible: !['downloading', 'downloaded'].includes(updaterStatus.code),
          enabled: !['downloading', 'downloaded'].includes(updaterStatus.code),
          click: async (event) => {
            Manager.libraries.updater.update({userAction: true});
            Manager.sendToRegisteredRenderers('app-menu:choice', {id: event.id})
            self.analytics(event);
          }
        },
        {
          id: 'main/update-install',
          label: `Install v${updaterStatus.version} &Update`,
          visible: updaterStatus.code === 'downloaded',
          enabled: updaterStatus.code === 'downloaded',
          click: async (event) => {
            Manager.sendToRegisteredRenderers('app-menu:choice', {id: event.id})
            self.analytics(event);
            Manager.relaunch({force: true});
          }
        },
        { type: 'separator' },
        {
          id: 'main/preferences',
          label: '&Preferences',
          accelerator: 'CommandOrControl+,',
          visible: false,
          click: async (event) => {
            Manager.sendToRegisteredRenderers('app-menu:choice', {id: event.id})
            self.analytics(event);
          }
        },

        // Mac options
        !isMac ? null :
        [
          {
            id: 'main/services',
            label: 'Services',
            role: 'services',
            // submenu: []
          },
          { type: 'separator' },
          {
            id: 'main/hide',
            label: `&Hide ${app.name}`,
            accelerator: 'Command+H',
            selector: 'hide:',
            // click: async (event) => {
            //   self.analytics(event);
            // }
          },
          {
            id: 'main/hide-others',
            label: 'Hide &Others',
            accelerator: 'Command+Shift+H',
            selector: 'hideOtherApplications:',
            // click: async (event) => {
            //   self.analytics(event);
            // }
          },
          {
            id: 'main/show-all',
            label: '&Show All',
            selector: 'unhideAllApplications:',
            // click: async (event) => {
            //   self.analytics(event);
            // }
          },
          {
            type: 'separator',
          },
        ],

        {
          id: 'main/relaunch',
          label: '&Relaunch',
          accelerator: isMac ? 'Command+Option+R' : 'Ctrl+Shift+W',
          click: async (event) => {
            self.analytics(event);
            Manager.relaunch({force: true});
          }
        },
        {
          id: 'main/quit',
          label: '&Quit',
          accelerator: isMac ? 'Command+Q' : 'Alt+F4',
          click: async (event) => {
            self.analytics(event);
            Manager.quit({force: true});
          }
        },
      ]
    },

    // File
    {
      id: 'file',
      label: '&File',
      visible: false,
      submenu: [

      ]
    },

    // Edit
    {
      id: 'edit',
      label: '&Edit',
      submenu: [
        {
          id: 'edit/undo',
          label: '&Undo',
          accelerator: 'CommandOrControl+Z',
          selector: 'undo:'
        },
        {
          id: 'edit/redo',
          label: '&Redo',
          accelerator: isMac ? 'Shift+Command+Z' : 'Control+Y',
          selector: 'redo:'
        },
        { type: 'separator'},
        {
          id: 'edit/cut',
          label: 'Cu&t',
          accelerator: 'CommandOrControl+X',
          selector: 'cut:'
        },
        {
          id: 'edit/copy',
          label: '&Copy',
          accelerator: 'CommandOrControl+C',
          selector: 'copy:'
        },
        {
          id: 'edit/paste',
          label: '&Paste',
          accelerator: 'CommandOrControl+V',
          selector: 'paste:'
        },
        {
          id: 'edit/select-all',
          label: 'Select &All',
          accelerator: 'CommandOrControl+A',
          selector: 'selectAll:'
        },
        // { type: 'separator' },
        // {
        //   label: '&Find in Page',
        //   accelerator: 'Ctrl+F',
        //   click: async () => {
        //     showMainWindow();
        //     sendCommand('tab:find');
        //   }
        // },

      ]
    },

    // View
    {
      id: 'view',
      label: '&View',
      submenu: [
        // {
        //   id: 'view/toggle-tab-bar',
        //   role: 'toggleTabBar',
        //   enabled: false,
        //   visible: false,
        // },

        // Relaunch
        !resolvedDeveloper ? null :
        [
          {
            id: 'view/reload',
            label: 'Reload',
            accelerator: 'CommandOrControl+R',
            click: async (event) => {
              const mainWindow = Manager.window().get(1);
              if (mainWindow) {
                Manager.window().show(mainWindow.id);
                mainWindow.browserWindow.webContents.reload();
                self.analytics(event);
              }
            }
          },
          { type: 'separator' },
        ],
        {
          id: 'view/toggle-fullscreen',
          label: 'Toggle Full Screen',
          accelerator: isMac ? 'Ctrl+Command+F' : 'F11',
          click: async (event) => {
            const mainWindow = Manager.window().get(1);
            if (mainWindow) {
              mainWindow.browserWindow.setFullScreen(!mainWindow.browserWindow.isFullScreen())
              self.analytics(event);
            }
          }
        },

        // Dev tools
        !resolvedDeveloper ? null :
        [
          {
            id: 'view/toggle-developer-tools',
            label: 'Toggle Developer Tools',
            accelerator: isMac ? 'Command+Option+I' : 'Ctrl+Shift+I',
            click: async (event) => {
              const mainWindow = Manager.window().get(1);
              if (mainWindow) {
                Manager.window().show(mainWindow.id);
                mainWindow.browserWindow.webContents.toggleDevTools();
                self.analytics(event);
              }
            }
          }
        ],
      ]
    },

    // Shortcuts
    {
      id: 'shortcuts',
      label: '&Shortcuts',
      visible: false,
      submenu: [

      ]
    },

    // Window
    {
      id: 'window',
      label: '&Window',
      submenu: [
        {
          id: 'window/minimize',
          label: 'Minimize',
          accelerator: 'CommandOrControl+M',
          selector: 'performMiniaturize:'
        },
        { type: 'separator' },
        {
          id: 'window/bring-all-to-front',
          label: 'Bring All to Front',
          selector: 'arrangeInFront:'
        }
      ]
    },

    // Help
    {
      id: 'help',
      label: '&Help',
      role: 'help',
      submenu: [
        {
          id: 'help/website',
          label: `${app.name} Home`,
          click: async (event) => {
            shell.openExternal(Manager.package.homepage);
            self.analytics(event);
          },
        },
        {
          id: 'help/discord',
          label: `${app.name} Support Discord`,
          click: async (event) => {
            shell.openExternal(`${Manager.package.homepage}/discord`);
            self.analytics(event);
          },
        },
      ]
    },

    // Dev
    !resolvedDeveloper ? null : {
      id: 'development',
      label: '&Development',
      submenu: [
        {
          id: 'development/open-exe-folder',
          label: 'Open exe folder',
          click: async (event) => {
            shell.showItemInFolder(app.getPath('exe'))
            self.analytics(event);
          },
        },        
        {
          id: 'development/open-user-data',
          label: 'Open user data folder',
          click: async (event) => {
            shell.showItemInFolder(app.getPath('userData'))
            self.analytics(event);
          },
        },
        {
          id: 'development/open-logs',
          label: 'Open logs folder',
          click: async (event) => {
            shell.showItemInFolder(app.getPath('logs'))
            self.analytics(event);
          },
        },        
      ]
    },

  ];

  self.dedupe();

  return self;
};

module.exports = Menu;
