function Renderer(Manager) {
  const self = this;
  self.initialized = false;
  self.webviews = [];
  self.setupAuthChangeListener = false;
  self.Manager = Manager;
  self._webviewId = 0;
}

Renderer.prototype.init = function (options) {
  const self = this;

  return self;
};

Renderer.prototype.onDOMContentLoaded = function (fn) {
  const self = this;
  if (
      ['interactive', 'complete'].includes(document.readyState)
  ) {
    fn();
  } else {
    document.addEventListener('DOMContentLoaded', function () {
      fn()
    });
  }
  return self;
};

Renderer.prototype.webview = function () {
  const self = this;
  return {
    create: function (options) {
      return new Promise(async function(resolve, reject) {
        const Manager = self.Manager;
        const powertools = Manager.require('node-powertools');

        options = options || {};
        options.parent = options.parent || document.body;
        options.position = options.position || 'beforeend';
        options.openDevTools = typeof options.openDevTools === 'undefined' ? self.Manager.isDevelopment : false
        options.setupAuthenticationHandler = typeof options.setupAuthenticationHandler === 'undefined' ? false : options.setupAuthenticationHandler
        options.authenticationHandlerDestination = typeof options.authenticationHandlerDestination === 'undefined' ? options.url : options.authenticationHandlerDestination
        options.fullscreen = typeof options.fullscreen === 'undefined' ? false : options.fullscreen

        options.id = options.id || `webview-${self._webviewId++}`;
        options.url = options.url || 'https://google.com';
        options.style = options.style || (options.fullscreen ? 'display:inline-flex; width:100vw; height:100vh' : 'display:inline-flex;');
        options.preload = typeof options.preload === 'undefined' ? '' : options.preload
        options.httpreferrer = typeof options.httpreferrer === 'undefined' ? '' : options.httpreferrer
        options.useragent = typeof options.useragent === 'undefined' ? '' : options.useragent
        options.partition = typeof options.partition === 'undefined' ? '' : options.partition
        options.webpreferences = typeof options.webpreferences === 'undefined' ? '' : options.webpreferences

        options.nodeintegration = typeof options.nodeintegration === 'undefined' ? false : options.nodeintegration
        options.nodeintegrationinsubframes = typeof options.nodeintegrationinsubframes === 'undefined' ? false : options.nodeintegrationinsubframes
        options.plugins = typeof options.plugins === 'undefined' ? false : options.plugins
        options.disablewebsecurity = typeof options.disablewebsecurity === 'undefined' ? false : options.disablewebsecurity
        options.allowpopups = typeof options.allowpopups === 'undefined' ? false : options.allowpopups


        const parent = options.parent;
        parent.insertAdjacentHTML(
          options.position,
          `<webview
            id="${options.id}"
            src="${options.url}"
            style="${options.style}"

            ${options.preload ? `preload="${options.preload}"` : ''}
            ${options.httpreferrer ? `httpreferrer="${options.httpreferrer}"` : ''}
            ${options.useragent ? `useragent="${options.useragent}"` : ''}
            ${options.partition ? `partition="${options.partition}"` : ''}
            ${options.webpreferences ? `webpreferences="${options.webpreferences}"` : ''}

            ${options.nodeintegration ? 'nodeintegration' : ''}
            ${options.nodeintegrationinsubframes ? 'nodeintegrationinsubframes' : ''}
            ${options.plugins ? 'plugins' : ''}
            ${options.disablewebsecurity ? 'disablewebsecurity' : ''}
            ${options.allowpopups ? 'allowpopups' : ''}

            ${options.setupAuthenticationHandler ? 'setupAuthenticationHandler' : ''}
            ${options.authenticationHandlerDestination ? `authenticationHandlerDestination="${options.authenticationHandlerDestination}"` : ''}
          ></webview>`
        )
        const webview = document.getElementById(options.id);

        webview.addEventListener('dom-ready', function () {
          webview.blur();
          setTimeout(function () {
            document.body.focus();
            setTimeout(function () {
              webview.focus();
              return resolve(webview);
            }, 10);
          }, 10);
        })

        // console.log('---options.url', options.url);

        // if (options.setupAuthenticationHandler) {
        //   webview.addEventListener('will-navigate', function (event) {
        //     const newUrl = new URL(event.url);
        //     const match = newUrl.pathname.match(/signin|signup|forgot/igm);
        //     const homepageUrl = new URL(options.url);
        //
        //     // if (Manager.isDevelopment && Manager.storage.electronManager.get('data.current.argv', {}).useDevelopmentURLs !== 'false') {
        //     //   homepageUrl.protocol = 'http:'
        //     //   homepageUrl.host = 'localhost:4000'
        //     // }
        //
        //     // console.log('======homepageUrl', homepageUrl);
        //     // console.log('======homepageUrl.toString()', homepageUrl.toString());
        //     console.log('+++will-navigate', newUrl.toString());
        //     if (newUrl.host !== homepageUrl.host) {
        //       event.preventDefault()
        //     } else if (match) {
        //       event.preventDefault()
        //       if (match[0] === 'signin') {
        //         self.openExternalAuthPage('signin');
        //       } else {
        //         self.openExternalAuthPage('signup');
        //       }
        //       homepageUrl.pathname = 'authentication/token'
        //       homepageUrl.searchParams.set('signout', true)
        //       webview.loadURL(homepageUrl.toString())
        //       .catch(e => console.error)
        //     }
        //   })
        // }
        if (options.setupAuthenticationHandler) {
          // console.log('------1');
          webview.addEventListener('did-navigate', async function (event) {
            const newUrl = new URL(event.url);
            const isInAppAuthenticatorPage = newUrl.pathname.startsWith('/authentication/');
            // const isAnyAuthPage = newUrl.pathname.match(/signin|signup|forgot/igm) && !isInAppAuthenticatorPage;
            const isAnyAuthPage = newUrl.pathname.match(/signin|signup|forgot/igm);
            const isWebsitesAuthPage = isAnyAuthPage && !isInAppAuthenticatorPage;
            const signInToken = newUrl.searchParams.get('token') || null
            const homepageUrl = new URL(options.url);
            // console.log('------2 newUrl', newUrl.toString());
            // console.log('------3 homepageUrl', homepageUrl.toString());

            // console.log('+++did-navigate', newUrl.toString());
            // console.log('+++isInAppAuthenticatorPage', isInAppAuthenticatorPage);
            // console.log('+++isAnyAuthPage', isAnyAuthPage);
            // console.log('+++isWebsitesAuthPage', isWebsitesAuthPage);
            // console.log('+++signInToken', signInToken);

            function _nav(url) {
              // console.log('+++ _nav to START', url);
              webview.stop();
              webview.loadURL(url)
              // .then(r => {console.log('+++ _nav to SUCCESS', url)})
              .catch(e => console.error)
            }

            // console.log('+++ 1');
            if (newUrl.host !== homepageUrl.host) {
              // console.log('------4');
              event.preventDefault();
              _nav(homepageUrl.toString())
            } else {
              // console.log('------5');
              // console.log('+++ 2 b');
              if (isAnyAuthPage) {
                // console.log('------6');
                // console.log('+++ 3');
                // homepageUrl.pathname = 'authentication/app'
                homepageUrl.pathname = 'authentication/required'
                homepageUrl.searchParams.set('signout', true)
                homepageUrl.searchParams.set('inApp', true)
                await firebase.auth()
                .signOut()
                .catch(e => console.error)
                if (isInAppAuthenticatorPage) {
                  // console.log('+++ 4 a', isAnyAuthPage[0]);
                  self.openExternalAuthPage(isAnyAuthPage[0]);
                  _nav(homepageUrl.toString())
                } else {
                  // console.log('+++ 4 b');
                  _nav(homepageUrl.toString())
                }
              }

            }
          })
          webview.addEventListener('page-title-updated', function (event, title) {
            document.title = event.title;
          })
        }

        await powertools.poll(function() {
          try {
            webview.getURL();
            return true;
          } catch (e) {
            return false;
          }
        }, {
          timeout: 60000
        })
        .catch(e => {
        })

        if (options.openDevTools) {
          webview.openDevTools({ mode: 'undocked', activate: false });
        }

        self.webviews.push({
          id: webview.getWebContentsId(),
          webview: webview,
          options: options,
        })

        // if (options.setupAuthenticationHandler) {
        //   if (!self.setupAuthChangeListener) {
        //     firebase.auth().onAuthStateChanged(function () {
        //       const auth = self.Manager.libraries.webManager.getAuth();
        //       const dom = self.Manager.libraries.webManager.dom();
        //
        //       console.log('+++onAuthStateChanged');
        //
        //       dom.select('webview[setupAuthenticationHandler]')
        //       .each((el) => {
        //         const originalUrl = el.getAttribute('authenticationHandlerDestination');
        //         console.log('------urlllll()', el.getURL(), originalUrl);
        //         // const url = new URL(el.getURL());
        //         const url = new URL(originalUrl);
        //
        //         function _nav(url) {
        //           // el.loadURL(url)
        //           // .catch(e => console.error)
        //         }
        //
        //         url.pathname = 'authentication/token'
        //         if (auth.user.auth.uid && auth.info.signInToken) {
        //           url.searchParams.set('token', auth.info.signInToken)
        //           url.searchParams.set('destination', originalUrl)
        //         } else {
        //           url.searchParams.set('signout', true)
        //         }
        //         // console.log('+++WV 2', url);
        //         // webview.loadURL(`https://slapform.com/authentication/token?token=${info.signInToken}&destination=https://slapform.com/dashboard`)
        //
        //         _nav(url.toString())
        //       })
        //     })
        //     self.setupAuthChangeListener = true;
        //   }
        // }
        if (options.setupAuthenticationHandler && self.Manager.libraries.webManager !== false) {
          self.Manager.libraries.webManager.refreshWebviewAuths()
        }

        return webview;

      });
    },
    get: function (id) {
      if (id) {
        return self.webviews.filter(w => w.id === id)[0];
      } else {
        return self.webviews;
      }
    },
    navigate: function (id, url, options) {
      return new Promise(function(resolve, reject) {
        const item = self.webview().get(id)
        if (item && item.webview) {
          return item.webview.loadURL(url, options)
        } else {
          reject(new Error(`Webview with id=${id} does not exist`))
        }
      });
    },
  }
};

Renderer.prototype.openExternalAuthPage = function (pathname, options) {
  const self = this;
  const Manager = self.Manager;

  pathname = `authentication/${pathname || 'signin'}`;
  options = options || {};
  // options.inApp = typeof options.inApp === 'undefined' ? true : options.inApp;
  options.provider = typeof options.provider === 'undefined' ? '' : options.provider;

  const newUrl = new URL(Manager.package.homepage)
  if (
    Manager.isDevelopment
    && Manager.storage.electronManager.get('data.current.argv', {}).useDevelopmentURLs !== 'false'
  ) {
    newUrl.protocol = 'http:'
    newUrl.host = 'localhost:4000'
  }
  newUrl.pathname = pathname;
  newUrl.searchParams.set('provider', options.provider || '')
  // newUrl.searchParams.set('inApp', options.inApp)
  newUrl.searchParams.set('source', 'app')
  newUrl.searchParams.set('signout', true)
  newUrl.searchParams.set('cb', new Date().getTime())
  newUrl.searchParams.set('destination', `${Manager.options.app.id}://electron-manager?command=user:authenticate`)

  const finalUrl = newUrl.toString();

  Manager.libraries.electron.shell.openExternal(finalUrl)
  return finalUrl;
}

Renderer.prototype.processDeepLink = function (url) {
  const self = this;
  return new Promise(function(resolve, reject) {
    self.Manager.libraries.electron.ipcRenderer.invoke('electron-manager-message', {command: 'main:process-deep-link', payload: {
      url: url
    }})
    .then(r => {
      return resolve(r);
    })

  });
};

module.exports = Renderer;
